@@ -7,7 +7,6 @@ from gaphor.diagram.presentation import (
 )
 from gaphor.diagram.shapes import (
     Box,
-    EditableText,
     Text,
     TextAlign,
     VerticalAlign,
@@ -16,6 +15,7 @@ from gaphor.diagram.shapes import (
 )
 from gaphor.diagram.support import represents
 from gaphor.diagram.text import FontStyle, FontWeight
+from gaphor.RAAML import raaml
 from gaphor.SysML.sysml import Block
 from gaphor.UML.classes.klass import (
     attribute_watches,
@@ -27,6 +27,10 @@ from gaphor.UML.umlfmt import format_property
 
 
 @represents(Block)
+@represents(raaml.Situation)
+@represents(raaml.Loss)
+@represents(raaml.Hazard)
+@represents(raaml.ControlStructure)
 class BlockItem(ElementPresentation[Block], Classified):
     def __init__(self, diagram, id=None):
         super().__init__(diagram, id)
@@ -51,14 +55,25 @@ class BlockItem(ElementPresentation[Block], Classified):
 
     show_references: attribute[int] = attribute("show_references", int, default=False)
 
+    def additional_stereotypes(self):
+        if isinstance(self.subject, raaml.Situation):
+            return ["Situation"]
+        elif isinstance(self.subject, raaml.ControlStructure):
+            return ["ControlStructure"]
+        elif isinstance(self.subject, Block):
+            return ["block"]
+        else:
+            return ()
+
     def update_shapes(self, event=None):
         self.shape = Box(
             Box(
                 Text(
-                    text=lambda: stereotypes_str(self.subject, ["block"]),
-                    style={"min-width": 0, "min-height": 0},
+                    text=lambda: stereotypes_str(
+                        self.subject, self.additional_stereotypes()
+                    )
                 ),
-                EditableText(
+                Text(
                     text=lambda: self.subject.name or "",
                     width=lambda: self.width - 4,
                     style={
@@ -70,7 +85,7 @@ class BlockItem(ElementPresentation[Block], Classified):
                 ),
                 Text(
                     text=lambda: from_package_str(self),
-                    style={"font-size": 10, "min-width": 0, "min-height": 0},
+                    style={"font-size": "x-small"},
                 ),
                 style={"padding": (12, 4, 12, 4)},
             ),
@@ -98,8 +113,6 @@ class BlockItem(ElementPresentation[Block], Classified):
             ),
             *(self.show_stereotypes and stereotype_compartments(self.subject) or []),
             style={
-                "min-width": 100,
-                "min-height": 50,
                 "vertical-align": VerticalAlign.TOP,
             },
             draw=draw_border,
@@ -115,7 +128,7 @@ class BlockItem(ElementPresentation[Block], Classified):
                 text=name,
                 style={
                     "padding": (0, 0, 4, 0),
-                    "font-size": 10,
+                    "font-size": "x-small",
                     "font-style": FontStyle.ITALIC,
                 },
             ),
@@ -124,6 +137,6 @@ class BlockItem(ElementPresentation[Block], Classified):
                 for attribute in self.subject.ownedAttribute
                 if predicate(attribute)
             ),
-            style={"padding": (4, 4, 4, 4), "min-height": 8},
+            style={"padding": (4, 4, 4, 4)},
             draw=draw_top_separator,
         )
