@@ -19,7 +19,7 @@ class SysMLModelingLanguage(ModelingLanguage):
         return sysml_toolbox_actions
 
     def lookup_element(self, name):
-        return getattr(sysml, name, None)
-
-    def lookup_diagram_item(self, name):
-        return getattr(diagramitems, name, None)
+        element_type = getattr(sysml, name, None)
+        if not element_type:
+            element_type = getattr(diagramitems, name, None)
+        return element_type
