@@ -75,6 +75,13 @@ classes = ToolSection(
             ),
             handle_index=SE,
         ),
+        ToolDef(
+            "toolbox-containment",
+            gettext("Containment"),
+            "gaphor-containment-symbolic",
+            "<Shift>C",
+            new_item_factory(diagramitems.ContainmentItem),
+        ),
         ToolDef(
             "toolbox-composite-association",
             gettext("Composite Association"),
@@ -119,11 +126,47 @@ classes = ToolSection(
             new_item_factory(diagramitems.GeneralizationItem),
         ),
         ToolDef(
-            "toolbox-implementation",
-            gettext("Implementation"),
-            "gaphor-implementation-symbolic",
+            "toolbox-interface-realization",
+            gettext("Interface Realization"),
+            "gaphor-interface-realization-symbolic",
             "<Shift>I",
-            new_item_factory(diagramitems.ImplementationItem),
+            new_item_factory(diagramitems.InterfaceRealizationItem),
+        ),
+        ToolDef(
+            "toolbox-data-type",
+            gettext("DataType"),
+            "gaphor-data-type-symbolic",
+            "<Shift>L",
+            new_item_factory(
+                diagramitems.DataTypeItem,
+                UML.DataType,
+                config_func=namespace_config,
+            ),
+            handle_index=SE,
+        ),
+        ToolDef(
+            "toolbox-enumeration",
+            gettext("Enumeration"),
+            "gaphor-enumeration-symbolic",
+            "<Shift>W",
+            new_item_factory(
+                diagramitems.EnumerationItem,
+                UML.Enumeration,
+                config_func=namespace_config,
+            ),
+            handle_index=SE,
+        ),
+        ToolDef(
+            "toolbox-primitive",
+            gettext("Primitive"),
+            "gaphor-primitive-symbolic",
+            "<Shift>H",
+            new_item_factory(
+                diagramitems.DataTypeItem,
+                UML.PrimitiveType,
+                config_func=namespace_config,
+            ),
+            handle_index=SE,
         ),
     ),
 )
