@@ -15,13 +15,7 @@ from gaphor.diagram.presentation import (
     ElementPresentation,
     from_package_str,
 )
-from gaphor.diagram.shapes import (
-    Box,
-    EditableText,
-    Text,
-    draw_border,
-    draw_top_separator,
-)
+from gaphor.diagram.shapes import Box, Text, draw_border, draw_top_separator
 from gaphor.diagram.support import represents
 from gaphor.UML.classes.stereotype import stereotype_compartments
 
@@ -75,7 +69,7 @@ class ClassItem(ElementPresentation[UML.Class], Classified):
                         self.subject, self.additional_stereotypes()
                     ),
                 ),
-                EditableText(
+                Text(
                     text=lambda: self.subject.name or "",
                     style={
                         "font-weight": FontWeight.BOLD,
@@ -86,7 +80,7 @@ class ClassItem(ElementPresentation[UML.Class], Classified):
                 ),
                 Text(
                     text=lambda: from_package_str(self),
-                    style={"font-size": 10, "min-width": 0, "min-height": 0},
+                    style={"font-size": "x-small"},
                 ),
                 style={"padding": (12, 4, 12, 4)},
             ),
@@ -104,8 +98,6 @@ class ClassItem(ElementPresentation[UML.Class], Classified):
             ),
             *(self.show_stereotypes and stereotype_compartments(self.subject) or []),
             style={
-                "min-width": 100,
-                "min-height": 50,
                 "vertical-align": VerticalAlign.TOP,
             },
             draw=draw_border,
@@ -148,19 +140,13 @@ def operation_watches(presentation, cast):
     ).watch(
         f"subject[{cast}].ownedOperation.visibility"
     ).watch(
-        f"subject[{cast}].ownedOperation.returnResult.lowerValue"
+        f"subject[{cast}].ownedOperation.ownedParameter.lowerValue"
     ).watch(
-        f"subject[{cast}].ownedOperation.returnResult.upperValue"
+        f"subject[{cast}].ownedOperation.ownedParameter.upperValue"
     ).watch(
-        f"subject[{cast}].ownedOperation.returnResult.typeValue"
+        f"subject[{cast}].ownedOperation.ownedParameter.typeValue"
     ).watch(
-        f"subject[{cast}].ownedOperation.formalParameter.lowerValue"
-    ).watch(
-        f"subject[{cast}].ownedOperation.formalParameter.upperValue"
-    ).watch(
-        f"subject[{cast}].ownedOperation.formalParameter.typeValue"
-    ).watch(
-        f"subject[{cast}].ownedOperation.formalParameter.defaultValue"
+        f"subject[{cast}].ownedOperation.ownedParameter.defaultValue"
     )
 
 
@@ -193,7 +179,7 @@ def attributes_compartment(subject):
             for attribute in subject.ownedAttribute
             if not attribute.association
         ),
-        style={"padding": (4, 4, 4, 4), "min-height": 8},
+        style={"padding": (4, 4, 4, 4)},
         draw=draw_top_separator,
     )
 
@@ -220,6 +206,6 @@ def operations_compartment(subject):
             )
             for operation in subject.ownedOperation
         ),
-        style={"padding": (4, 4, 4, 4), "min-height": 8},
+        style={"padding": (4, 4, 4, 4)},
         draw=draw_top_separator,
     )
