@@ -1,6 +1,12 @@
 from gaphor import UML
 from gaphor.core import transactional
-from gaphor.diagram.propertypages import PropertyPageBase, PropertyPages, new_builder
+from gaphor.diagram.propertypages import (
+    PropertyPageBase,
+    PropertyPages,
+    new_resource_builder,
+)
+
+new_builder = new_resource_builder("gaphor.UML.components")
 
 
 @PropertyPages.register(UML.Component)
@@ -19,14 +25,14 @@ class ComponentPropertyPage(PropertyPageBase):
         if not subject:
             return
 
-        builder = new_builder("component-editor")
+        builder = new_builder(
+            "component-editor",
+            signals={"indirectly-instantiated-changed": (self._on_ii_change,)},
+        )
 
         ii = builder.get_object("indirectly-instantiated")
         ii.set_active(subject.isIndirectlyInstantiated)
 
-        builder.connect_signals(
-            {"indirectly-instantiated-changed": (self._on_ii_change,)}
-        )
         return builder.get_object("component-editor")
 
     @transactional
