@@ -8,22 +8,24 @@ import logging
 import uuid
 from dataclasses import dataclass
 from functools import lru_cache
-from typing import TYPE_CHECKING, Iterable, Iterator, Optional, Sequence, Set, Union
+from typing import Iterable, Iterator, Sequence, TypeVar
 
 import gaphas
+from typing_extensions import Protocol, runtime_checkable
 
 from gaphor.core.modeling.collection import collection
-from gaphor.core.modeling.coremodel import Element, PackageableElement
-from gaphor.core.modeling.element import Id, RepositoryProtocol
-from gaphor.core.modeling.event import AssociationDeleted, DiagramItemCreated
+from gaphor.core.modeling.element import Element, Id, RepositoryProtocol
+from gaphor.core.modeling.event import AssociationDeleted
 from gaphor.core.modeling.presentation import Presentation
-from gaphor.core.modeling.properties import association, relation_many, relation_one
+from gaphor.core.modeling.properties import (
+    association,
+    attribute,
+    relation_many,
+    relation_one,
+)
 from gaphor.core.modeling.stylesheet import StyleSheet
 from gaphor.core.styling import Style, StyleNode
 
-if TYPE_CHECKING:
-    from gaphor.UML import Package
-
 log = logging.getLogger(__name__)
 
 
@@ -33,7 +35,6 @@ FALLBACK_STYLE: Style = {
     "color": (0, 0, 0, 1),
     "font-family": "sans",
     "font-size": 14,
-    "highlight-color": (0, 0, 1, 0.4),
     "line-width": 2,
     "padding": (0, 0, 0, 0),
 }
@@ -118,7 +119,7 @@ def attrstr(obj):
 
 class StyledDiagram:
     def __init__(
-        self, diagram: Diagram, selection: Optional[gaphas.view.Selection] = None
+        self, diagram: Diagram, selection: gaphas.view.Selection | None = None
     ):
         self.diagram = diagram
         self.selection = selection or gaphas.view.Selection()
@@ -147,22 +148,22 @@ class StyledDiagram:
 class StyledItem:
     """Wrapper to allow style information to be retrieved.
 
-    For convenience, a view can be added. The view will provide pseudo-
-    classes for the item (focus, hover, etc.)
+    For convenience, a selection can be added. The selection instance
+    will provide pseudo-classes for the item (focus, hover, etc.).
     """
 
     def __init__(
-        self, item: Presentation, selection: Optional[gaphas.view.Selection] = None
+        self, item: Presentation, selection: gaphas.view.Selection | None = None
     ):
         assert item.diagram
         self.item = item
         self.diagram = item.diagram
-        self.selection = selection or gaphas.view.Selection()
+        self.selection = selection
 
     def name(self) -> str:
         return removesuffix(type(self.item).__name__, "Item").lower()
 
-    def parent(self) -> Union[StyledItem, StyledDiagram]:
+    def parent(self) -> StyledItem | StyledDiagram:
         parent = self.item.parent
         return (
             StyledItem(parent, self.selection)
@@ -185,33 +186,25 @@ class StyledItem:
         item = self.item
         selection = self.selection
         return (
-            "active" if item in selection.selected_items else "",
-            "focus" if item is selection.focused_item else "",
-            "hover" if item is selection.hovered_item else "",
-            "drop" if item is selection.dropzone_item else "",
+            (
+                "active" if item in selection.selected_items else "",
+                "focus" if item is selection.focused_item else "",
+                "hover" if item is selection.hovered_item else "",
+                "drop" if item is selection.dropzone_item else "",
+                "disabled" if item in selection.grayed_out_items else "",
+            )
+            if selection
+            else ()
         )
 
 
-class PseudoCanvas:
-    """A "pseudo" canvas implementation, for storing items."""
-
-    def __init__(self, diagram: Diagram):
-        self.diagram = diagram
-
-    def save(self, save_func):
-        for item in self.diagram.ownedPresentation:
-            if not item.parent:
-                save_func(item)
-
-
-class Diagram(PackageableElement):
+class Diagram(Element):
     """Diagrams may contain model elements and can be owned by a Package."""
 
-    package: relation_one[Package]
+    name: attribute[str]
+    element: relation_one[Element]
 
-    def __init__(
-        self, id: Optional[Id] = None, model: Optional[RepositoryProtocol] = None
-    ):
+    def __init__(self, id: Id | None = None, model: RepositoryProtocol | None = None):
         """Initialize the diagram with an optional id and element model.
 
         The diagram also has a canvas.
@@ -221,15 +214,11 @@ class Diagram(PackageableElement):
         self._connections = gaphas.connections.Connections()
         self._connections.add_handler(self._on_constraint_solved)
 
-        self._registered_views: Set[gaphas.view.model.View] = set()
+        self._registered_views: set[gaphas.view.model.View] = set()
 
         self._watcher = self.watcher()
         self._watcher.watch("ownedPresentation", self._presentation_removed)
 
-        # Record all items changed during constraint solving,
-        # so their `post_update()` method can be called.
-        self._resolved_items: Set[gaphas.item.Item] = set()
-
     ownedPresentation: relation_many[Presentation] = association(
         "ownedPresentation", Presentation, composite=True, opposite="diagram"
     )
@@ -239,21 +228,17 @@ class Diagram(PackageableElement):
             self._update_views(removed_items=(event.old_value,))
 
     @property
-    def styleSheet(self) -> Optional[StyleSheet]:
+    def styleSheet(self) -> StyleSheet | None:
         return next(self.model.select(StyleSheet), None)
 
     def style(self, node: StyleNode) -> Style:
         style_sheet = self.styleSheet
-        return {
-            **FALLBACK_STYLE,  # type: ignore[misc]
-            **(style_sheet.match(node) if style_sheet else {}),
-        }
+        return style_sheet.match(node) if style_sheet else FALLBACK_STYLE
 
     def save(self, save_func):
         """Apply the supplied save function to this diagram and the canvas."""
 
         super().save(save_func)
-        save_func("canvas", PseudoCanvas(self))
 
     def postload(self):
         """Handle post-load functionality for the diagram."""
@@ -271,17 +256,10 @@ class Diagram(PackageableElement):
         return self.create_as(type, str(uuid.uuid1()), parent, subject)
 
     def create_as(self, type, id, parent=None, subject=None):
-        if not (type and issubclass(type, Presentation)):
-            raise TypeError(
-                f"Type {type} can not be added to a diagram as it is not a diagram item"
-            )
-        # Avoid events that reference this element before its created-event is emitted.
-        with self.model.block_events():
-            item = type(diagram=self, id=id)
-        assert isinstance(
-            item, gaphas.Item
-        ), f"Type {type} does not comply with Item protocol"
-        self.model.handle(DiagramItemCreated(self, item))
+        assert isinstance(self.model, PresentationRepositoryProtocol)
+        item = self.model.create_as(type, id, diagram=self)
+        if not isinstance(item, gaphas.Item):
+            raise TypeError(f"Type {type} does not comply with Item protocol")
         if subject:
             item.subject = subject
         if parent:
@@ -327,7 +305,7 @@ class Diagram(PackageableElement):
                 yield root
                 yield from iter_children(root)
 
-    def get_parent(self, item: Presentation) -> Optional[Presentation]:
+    def get_parent(self, item: Presentation) -> Presentation | None:
         return item.parent
 
     def get_children(self, item: Presentation) -> Iterable[Presentation]:
@@ -337,20 +315,13 @@ class Diagram(PackageableElement):
         items_set = set(items)
         return (n for n in self.get_all_items() if n in items_set)
 
-    def request_update(
-        self, item: gaphas.item.Item, update: bool = True, matrix: bool = True
-    ) -> None:
-        if update and matrix:
-            self._update_views(dirty_items=(item,), dirty_matrix_items=(item,))
-        elif update:
-            self._update_views(dirty_items=(item,))
-        elif matrix:
-            self._update_views(dirty_matrix_items=(item,))
+    def request_update(self, item: gaphas.item.Item) -> None:
+        self._update_views(dirty_items=(item,))
 
-    def _update_views(self, dirty_items=(), dirty_matrix_items=(), removed_items=()):
+    def _update_views(self, dirty_items=(), removed_items=()):
         """Send an update notification to all registered views."""
         for v in self._registered_views:
-            v.request_update(dirty_items, dirty_matrix_items, removed_items)
+            v.request_update(dirty_items, removed_items)
 
     @gaphas.decorators.nonrecursive
     def update_now(
@@ -359,7 +330,6 @@ class Diagram(PackageableElement):
         dirty_matrix_items: Sequence[Presentation] = (),
     ) -> None:
         """Update the diagram canvas."""
-
         sort = self.sort
 
         def dirty_items_with_ancestors():
@@ -368,38 +338,22 @@ class Diagram(PackageableElement):
                 yield from gaphas.canvas.ancestors(self, item)
 
         all_dirty_items = list(reversed(list(sort(dirty_items_with_ancestors()))))
-        contexts = self._pre_update_items(all_dirty_items)
-
-        self._resolved_items.clear()
+        self._update_items(all_dirty_items)
 
         self._connections.solve()
 
-        all_dirty_items.extend(self._resolved_items)
-        self._post_update_items(reversed(list(sort(all_dirty_items))), contexts)
-
-    def _pre_update_items(self, items):
-        contexts = {}
+    def _update_items(self, items):
         for item in items:
-            context = UpdateContext(style=self.style(StyledItem(item)))
-            item.pre_update(context)
-            contexts[item] = context
-        return contexts
-
-    def _post_update_items(self, items, contexts):
-        for item in items:
-            context = contexts.get(item)
-            if not context:
-                context = UpdateContext(style=self.style(StyledItem(item)))
-            item.post_update(context)
+            update = getattr(item, "update", None)
+            if update:
+                update(UpdateContext(style=self.style(StyledItem(item))))
 
     def _on_constraint_solved(self, cinfo: gaphas.connections.Connection) -> None:
         dirty_items = set()
         if cinfo.item:
             dirty_items.add(cinfo.item)
-            self._resolved_items.add(cinfo.item)
         if cinfo.connected:
             dirty_items.add(cinfo.connected)
-            self._resolved_items.add(cinfo.connected)
         if dirty_items:
             self._update_views(dirty_items)
 
@@ -413,3 +367,12 @@ class Diagram(PackageableElement):
 Presentation.diagram = association(
     "diagram", Diagram, upper=1, opposite="ownedPresentation"
 )
+
+
+P = TypeVar("P", bound=Presentation)
+
+
+@runtime_checkable
+class PresentationRepositoryProtocol(Protocol):
+    def create_as(self, type: type[P], id: str, diagram: Diagram) -> P:
+        ...
