@@ -47,14 +47,14 @@ Style = TypedDict(
         "dash-style": Sequence[Number],
         "padding": Padding,
         "font-family": str,
-        "font-size": Number,
+        "font-size": Union[int, float, str],
         "font-style": FontStyle,
         "font-weight": FontWeight,
-        "highlight-color": Color,
         "line-style": Number,
         "line-width": Number,
         "min-width": Number,
         "min-height": Number,
+        "opacity": Number,
         "text-decoration": TextDecoration,
         "text-align": TextAlign,
         "text-color": Color,
