@@ -5,7 +5,7 @@ We bind the Toolbox to a diagram. When a diagram page (tab) is switched,
 the actions bound to the toolbuttons should change as well.
 """
 
-from typing import Callable, NamedTuple, Optional, Sequence
+from typing import Callable, NamedTuple, Optional, Sequence, Tuple
 
 from gaphas.item import SE
 
@@ -14,15 +14,18 @@ from gaphor.core import gettext
 from gaphor.core.modeling import Diagram, Presentation
 from gaphor.diagram import general
 from gaphor.diagram.diagramtools import new_item_factory
+from gaphor.UML.modelfactory import owner_package
 
 ItemFactory = Callable[[Diagram, Optional[Presentation]], Presentation]
 
 
+def default_namespace(new_item):
+    new_item.subject.package = owner_package(new_item.diagram)
+
+
 def namespace_config(new_item):
-    subject = new_item.subject
-    diagram = new_item.diagram
-    subject.package = diagram.namespace
-    subject.name = f"New{type(subject).__name__}"
+    default_namespace(new_item)
+    new_item.subject.name = f"New{type(new_item.subject).__name__}"
 
 
 class ToolDef(NamedTuple):
@@ -42,6 +45,12 @@ class ToolSection(NamedTuple):
 ToolboxDefinition = Sequence[ToolSection]
 
 
+def tooliter(toolbox_actions: Sequence[Tuple[str, Sequence[ToolDef]]]):
+    """Iterate toolbox items, regardless of section headers."""
+    for name, section in toolbox_actions:
+        yield from section
+
+
 general_tools = ToolSection(
     gettext("General"),
     (
