@@ -1,22 +1,18 @@
 from gi.repository import Gtk
 
-from gaphor.core import transactional
 from gaphor.diagram.general.comment import CommentItem
 from gaphor.diagram.inlineeditors import InlineEditor, show_popover
+from gaphor.transaction import Transaction
 
 
 @InlineEditor.register(CommentItem)
-def CommentItemInlineEditor(item, view, pos=None) -> bool:
-    @transactional
+def CommentItemInlineEditor(item, view, event_manager, pos=None) -> bool:
     def update_text():
         text = buffer.get_text(
             buffer.get_start_iter(), buffer.get_end_iter(), include_hidden_chars=True
         )
-        item.subject.body = text
-        return True
-
-    def escape():
-        subject.body = body
+        with Transaction(event_manager):
+            item.subject.body = text
 
     subject = item.subject
     if not subject:
@@ -29,7 +25,6 @@ def CommentItemInlineEditor(item, view, pos=None) -> bool:
     startiter, enditer = buffer.get_bounds()
     buffer.move_mark_by_name("selection_bound", startiter)
     buffer.move_mark_by_name("insert", enditer)
-    buffer.connect("changed", lambda _: update_text())
 
     text_view = Gtk.TextView.new_with_buffer(buffer)
     box = view.get_item_bounding_box(view.selection.hovered_item)
@@ -40,5 +35,6 @@ def CommentItemInlineEditor(item, view, pos=None) -> bool:
     text_view.show()
     frame.show()
 
-    show_popover(frame, view, box, escape)
+    show_popover(frame, view, box, update_text)
+
     return True
