@@ -9,13 +9,14 @@ from gaphas.aspect.connector import ConnectionSink
 from gaphas.aspect.connector import Connector as ConnectorAspect
 from gaphas.connector import Handle
 from gaphas.geometry import Rectangle, distance_rectangle_point
+from gaphas.solver.constraint import BaseConstraint
 
 from gaphor.core.modeling.diagram import Diagram
 from gaphor.core.modeling.event import RevertibeEvent
 from gaphor.core.modeling.presentation import Presentation, S
 from gaphor.core.modeling.properties import attribute
-from gaphor.core.styling import Style
-from gaphor.diagram.shapes import combined_style, draw_highlight
+from gaphor.core.styling import Style, merge_styles
+from gaphor.diagram.shapes import stroke
 from gaphor.diagram.text import TextAlign, text_point_at_line
 
 
@@ -43,7 +44,7 @@ def from_package_str(item):
     if parent and parent.subject and parent.subject.namespace is not namespace:
         return False
 
-    return f"(from {namespace.name})" if namespace is not item.diagram.namespace else ""
+    return f"(from {namespace.name})" if namespace is not item.diagram.owner else ""
 
 
 def postload_connect(item: gaphas.Item, handle: gaphas.Handle, target: gaphas.Item):
@@ -99,17 +100,21 @@ class ElementPresentation(gaphas.Element, HandlePositionUpdate, Presentation[S])
     implement the method `update_shapes()` and set self.shape there.
     """
 
-    width: int
-    height: int
-
     _port_sides = ("top", "right", "bottom", "left")
 
-    def __init__(self, diagram: Diagram, id=None, shape=None):
-        super().__init__(connections=diagram.connections, diagram=diagram, id=id)  # type: ignore[call-arg]
+    def __init__(self, diagram: Diagram, id=None, shape=None, width=100, height=50):
+        super().__init__(connections=diagram.connections, diagram=diagram, id=id, width=width, height=height)  # type: ignore[call-arg]
         self._shape = shape
         for handle in self.handles():
             self.watch_handle(handle)
 
+        diagram.connections.add_constraint(
+            self, MinimalValueConstraint(self.min_width, width)
+        )
+        diagram.connections.add_constraint(
+            self, MinimalValueConstraint(self.min_height, height)
+        )
+
     def port_side(self, port):
         return self._port_sides[self._ports.index(port)]
 
@@ -123,15 +128,12 @@ class ElementPresentation(gaphas.Element, HandlePositionUpdate, Presentation[S])
         """Updating the shape configuration, e.g. when extra elements have to
         be drawn or when styling changes."""
 
-    def pre_update(self, context):
+    def update(self, context):
         if not self.shape:
             self.update_shapes()
         if self.shape:
             self.min_width, self.min_height = self.shape.size(context)
 
-    def post_update(self, context):
-        pass
-
     def draw(self, context):
         x, y = self.handles()[0].pos
         cairo = context.cairo
@@ -148,9 +150,7 @@ class ElementPresentation(gaphas.Element, HandlePositionUpdate, Presentation[S])
         super().save(save_func)
 
     def load(self, name, value):
-        if name == "matrix":
-            self.matrix.set(*ast.literal_eval(value))
-        elif name == "width":
+        if name == "width":
             self.width = ast.literal_eval(value)
         elif name == "height":
             self.height = ast.literal_eval(value)
@@ -162,6 +162,18 @@ class ElementPresentation(gaphas.Element, HandlePositionUpdate, Presentation[S])
         self.update_shapes()
 
 
+class MinimalValueConstraint(BaseConstraint):
+    def __init__(self, var, min):
+        super().__init__(var)
+        self._min = min
+
+    def solve_for(self, var):
+        min = self._min
+        if var is min:
+            return
+        var.value = max(var.value, min)
+
+
 class LinePresentation(gaphas.Line, HandlePositionUpdate, Presentation[S]):
     def __init__(
         self,
@@ -195,6 +207,10 @@ class LinePresentation(gaphas.Line, HandlePositionUpdate, Presentation[S]):
     orthogonal: attribute[int] = attribute("orthogonal", int, 0)
     horizontal: attribute[int] = attribute("horizontal", int, 0)
 
+    @property
+    def middle_shape_size(self) -> Rectangle:
+        return self._shape_middle_rect
+
     def insert_handle(self, index: int, handle: Handle) -> None:
         super().insert_handle(index, handle)
         self.watch_handle(handle)
@@ -203,10 +219,7 @@ class LinePresentation(gaphas.Line, HandlePositionUpdate, Presentation[S]):
         self.remove_watch_handle(handle)
         super().remove_handle(handle)
 
-    def pre_update(self, context):
-        pass
-
-    def post_update(self, context):
+    def update_shape_bounds(self, context):
         def shape_bounds(shape, align):
             if shape:
                 size = shape.size(context)
@@ -236,7 +249,6 @@ class LinePresentation(gaphas.Line, HandlePositionUpdate, Presentation[S]):
         def draw_line_end(end_handle, second_handle, draw):
             pos, p1 = end_handle.pos, second_handle.pos
             angle = atan2(p1.y - pos.y, p1.x - pos.x)
-            cr = context.cairo
             cr.save()
             try:
                 cr.translate(*pos)
@@ -245,15 +257,11 @@ class LinePresentation(gaphas.Line, HandlePositionUpdate, Presentation[S]):
             finally:
                 cr.restore()
 
-        style = combined_style(context.style, self.style)
+        style = merge_styles(context.style, self.style)
         context = replace(context, style=style)
 
+        self.update_shape_bounds(context)
         cr = context.cairo
-        cr.set_line_width(self.line_width)
-        cr.set_dash(style.get("dash-style", ()), 0)
-        stroke = style["color"]
-        if stroke:
-            cr.set_source_rgba(*stroke)
 
         handles = self._handles
         draw_line_end(handles[0], handles[1], self.draw_head)
@@ -263,8 +271,7 @@ class LinePresentation(gaphas.Line, HandlePositionUpdate, Presentation[S]):
 
         draw_line_end(handles[-1], handles[-2], self.draw_tail)
 
-        draw_highlight(context)
-        cr.stroke()
+        stroke(context)
 
         for shape, rect in (
             (self.shape_head, self._shape_head_rect),
@@ -289,9 +296,7 @@ class LinePresentation(gaphas.Line, HandlePositionUpdate, Presentation[S]):
         save_connection("tail-connection", self.tail)
 
     def load(self, name, value):
-        if name == "matrix":
-            self.matrix.set(*ast.literal_eval(value))
-        elif name == "points":
+        if name == "points":
             points = ast.literal_eval(value)
             for _ in range(len(points) - 2):
                 h = Handle((0, 0))
@@ -308,6 +313,8 @@ class LinePresentation(gaphas.Line, HandlePositionUpdate, Presentation[S]):
             super().load(name, value)
 
     def postload(self):
+        super().postload()
+
         if self.orthogonal:
             self._set_orthogonal(self.orthogonal)
 
@@ -321,8 +328,6 @@ class LinePresentation(gaphas.Line, HandlePositionUpdate, Presentation[S]):
             assert self._connections.get_connection(self.tail)
             del self._load_tail_connection
 
-        super().postload()
-
     def _on_orthogonal(self, event):
         self._set_orthogonal(event.new_value)
 
