@@ -2,26 +2,6 @@
 
 To register property pages implemented in this module, it is imported in
 gaphor.adapter package.
-
-# TODO: make all labels align top-left
-# Add hidden columns for list stores where i can put the actual object
-# being edited.
-
-TODO:
- - stereotypes
- - association / association ends.
- - Follow HIG guidelines:
-   * Leave a 12-pixel border between the edge of the window and
-     the nearest controls.
-   * Leave a 12-pixel horizontal gap between a control and its label. (The gap
-     may be bigger for other controls in the same group, due to differences in
-     the lengths of the labels.)
-   * Labels must be concise and make sense when taken out of context.
-     Otherwise, users relying on screenreaders or similar assistive
-     technologies will not always be able to immediately understand the
-     relationship between a control and those surrounding it.
-   * Assign access keys to all editable controls. Ensure that using the access
-     key focuses its associated control.
 """
 
 import abc
@@ -29,21 +9,34 @@ import importlib
 from typing import Callable, Dict, List, Tuple, Type
 
 import gaphas.item
+from gaphas.decorators import g_async
 from gaphas.segment import Segment
-from gi.repository import Gdk, Gtk
+from gi.repository import Gtk
 
 from gaphor.core import transactional
 from gaphor.core.modeling import Element
 
 
-def new_builder(*object_ids):
-    builder = Gtk.Builder()
-    builder.set_translation_domain("gaphor")
-    with importlib.resources.path(
-        "gaphor.diagram", "propertypages.glade"
-    ) as glade_file:
-        builder.add_objects_from_file(str(glade_file), object_ids)
-    return builder
+def new_resource_builder(package, property_pages="propertypages"):
+    def new_builder(*object_ids, signals=None):
+        if Gtk.get_major_version() == 3:
+            builder = Gtk.Builder()
+            ui_file = f"{property_pages}.glade"
+        else:
+            builder = Gtk.Builder(signals)
+            ui_file = f"{property_pages}.ui"
+
+        builder.set_translation_domain("gaphor")
+        with importlib.resources.path(package, ui_file) as glade_file:
+            builder.add_objects_from_file(str(glade_file), object_ids)
+        if signals and Gtk.get_major_version() == 3:
+            builder.connect_signals(signals)
+        return builder
+
+    return new_builder
+
+
+new_builder = new_resource_builder("gaphor.diagram")
 
 
 class _PropertyPages:
@@ -101,24 +94,34 @@ class EditableTreeModel(Gtk.ListStore):
     new values.
     """
 
-    def __init__(self, item, cols=None):
+    def __init__(self, item, cols):
         """Create new model.
 
         Args:
           item (Presentation): diagram item owning tree model
           cols (tuple): model column types, defaults to [str, object]
         """
-
-        if cols is None:
-            cols = (str, object)
         super().__init__(*cols)
         self._item = item
 
-        for data in self._get_rows():
+        for data in self.get_rows():
             self.append(data)
         self._add_empty()
+        self.connect_after("row-inserted", self.on_row_inserted)
 
-    def _get_rows(self):
+    def on_row_inserted(self, model, path, iter):
+        """This method is called when new elements are added and when a row is
+        moved via DnD."""
+        self._sync_model()
+
+    @g_async(single=True)
+    def _sync_model(self):
+        """Align the order of elements in the model with the order in the list
+        store."""
+        new_order = [row[-1] for row in self if row[-1]]
+        self.sync_model(new_order)
+
+    def get_rows(self):
         """Return rows to be edited.
 
         Last column has to contain object being edited.
@@ -126,15 +129,15 @@ class EditableTreeModel(Gtk.ListStore):
 
         raise NotImplementedError
 
-    def _create_object(self):
+    def create_object(self):
         """Create new object."""
         raise NotImplementedError
 
-    def _set_object_value(self, row, col, value):
+    def set_object_value(self, row, col, value):
         """Update row's column with a value."""
         raise NotImplementedError
 
-    def _swap_objects(self, o1, o2):
+    def swap_objects(self, o1, o2):
         """Swap two objects.
 
         If objects are swapped, then return ``True``.
@@ -157,7 +160,7 @@ class EditableTreeModel(Gtk.ListStore):
             return
         o1 = self[a][-1]
         o2 = self[b][-1]
-        if o1 and o2 and self._swap_objects(o1, o2):
+        if o1 and o2 and self.swap_objects(o1, o2):
             super().swap(a, b)
 
     def _add_empty(self):
@@ -174,13 +177,13 @@ class EditableTreeModel(Gtk.ListStore):
 
         elif col == 0 and value and not row[-1]:
             # create new object
-            obj = self._create_object()
+            obj = self.create_object()
             row[-1] = obj
-            self._set_object_value(row, col, value)
+            self.set_object_value(row, col, value)
             self._add_empty()
 
         elif row[-1]:
-            self._set_object_value(row, col, value)
+            self.set_object_value(row, col, value)
 
     def remove(self, iter):
         """Remove object from GTK model and destroy it."""
@@ -208,23 +211,6 @@ def on_bool_cell_edited(renderer, path, model, col):
     model.update(iter, col, renderer.get_active())
 
 
-@transactional
-def on_keypress_event(tree, event):
-    k = Gdk.keyval_name(event.keyval).lower()
-    if k in ("backspace", "delete"):
-        model, iter = tree.get_selection().get_selected()
-        if iter:
-            model.remove(iter)
-    elif k in ("equal", "plus"):
-        model, iter = tree.get_selection().get_selected()
-        model.swap(iter, model.iter_next(iter))
-        return True
-    elif k in ("minus", "underscore"):
-        model, iter = tree.get_selection().get_selected()
-        model.swap(iter, model.iter_previous(iter))
-        return True
-
-
 class ComboModel(Gtk.ListStore):
     """combo box model.
 
@@ -276,22 +262,23 @@ class LineStylePage(PropertyPageBase):
         self.horizontal_button: Gtk.Button
 
     def construct(self):
-        builder = new_builder("line-editor")
-
-        rectilinear_button = builder.get_object("line-rectilinear")
-        rectilinear_button.set_active(self.item.orthogonal)
-
-        horizontal_button = builder.get_object("flip-orientation")
-        horizontal_button.set_active(self.item.horizontal)
-        horizontal_button.set_sensitive(self.item.orthogonal)
-        self.horizontal_button = horizontal_button
-
-        builder.connect_signals(
-            {
+        builder = new_builder(
+            "line-editor",
+            signals={
                 "rectilinear-changed": (self._on_orthogonal_change,),
                 "orientation-changed": (self._on_horizontal_change,),
-            }
+            },
         )
+
+        rectilinear_button = builder.get_object("line-rectilinear")
+        horizontal_button = builder.get_object("flip-orientation")
+
+        self.horizontal_button = horizontal_button
+
+        rectilinear_button.set_active(self.item.orthogonal)
+        horizontal_button.set_active(self.item.horizontal)
+        horizontal_button.set_sensitive(self.item.orthogonal)
+
         return builder.get_object("line-editor")
 
     @transactional
@@ -308,3 +295,47 @@ class LineStylePage(PropertyPageBase):
     def _on_horizontal_change(self, button):
         self.item.horizontal = button.get_active()
         self.item.diagram.update_now((self.item,))
+
+
+@PropertyPages.register(Element)
+class NotePropertyPage(PropertyPageBase):
+    """A facility to add a little note/remark."""
+
+    order = 300
+
+    def __init__(self, subject):
+        self.subject = subject
+        self.watcher = subject and subject.watcher()
+
+    def construct(self):
+        subject = self.subject
+
+        if not subject:
+            return
+
+        builder = new_builder("note-editor")
+        text_view = builder.get_object("note")
+
+        buffer = Gtk.TextBuffer()
+        if subject.note:
+            buffer.set_text(subject.note)
+        text_view.set_buffer(buffer)
+
+        changed_id = buffer.connect("changed", self._on_body_change)
+
+        def handler(event):
+            if not text_view.props.has_focus:
+                buffer.handler_block(changed_id)
+                buffer.set_text(event.new_value or "")
+                buffer.handler_unblock(changed_id)
+
+        self.watcher.watch("note", handler)
+        text_view.connect("destroy", self.watcher.unsubscribe_all)
+
+        return builder.get_object("note-editor")
+
+    @transactional
+    def _on_body_change(self, buffer):
+        self.subject.note = buffer.get_text(
+            buffer.get_start_iter(), buffer.get_end_iter(), False
+        )
