@@ -1,20 +1,11 @@
 """Support classes for dealing with text."""
 
-from typing import Tuple, Union
-
-import gi
 from gaphas.canvas import instant_cairo_context
-from gaphas.geometry import Rectangle
-from gaphas.painter.boundingboxpainter import CairoBoundingBoxContext
 from gaphas.painter.freehand import FreeHandCairoContext
+from gi.repository import GLib, Pango, PangoCairo
 
 from gaphor.core.styling import FontStyle, FontWeight, Style, TextAlign, TextDecoration
 
-# fmt: off
-gi.require_version('PangoCairo', '1.0')  # noqa: isort:skip
-from gi.repository import GLib, Pango, PangoCairo  # noqa: isort:skip
-# fmt: on
-
 
 class Layout:
     def __init__(
@@ -122,33 +113,10 @@ class Layout:
 
         if isinstance(cr, FreeHandCairoContext):
             PangoCairo.show_layout(cr.cr, layout)
-        elif isinstance(cr, CairoBoundingBoxContext):
-            w, h = layout.get_pixel_size()
-            cr.rel_line_to(w, h)
-            cr.stroke()
         else:
             PangoCairo.show_layout(cr, layout)
 
 
-def focus_box_pos(
-    bounding_box: Rectangle,
-    text_size: Tuple[Union[float, int], Union[float, int]],
-    text_align: TextAlign,
-) -> Tuple[int, int]:
-    """Calculate the focus box position based on alignment style."""
-    x, y, width, height = bounding_box
-    w, h = text_size
-
-    if text_align is TextAlign.CENTER:
-        x += (width - w) / 2
-    elif text_align is TextAlign.RIGHT:
-        x += width - w
-
-    y += (height - h) / 2
-
-    return x, y
-
-
 def text_point_at_line(points, size, text_align):
     """Provide a position (x, y) to draw a text close to a line.
 
