@@ -23,13 +23,12 @@ from gaphor.core.modeling.event import (
     AssociationDeleted,
     AssociationSet,
     AttributeUpdated,
-    DiagramItemCreated,
-    DiagramItemDeleted,
     ElementCreated,
     ElementDeleted,
     ModelReady,
     RevertibeEvent,
 )
+from gaphor.core.modeling.presentation import Presentation
 from gaphor.core.modeling.properties import association as association_property
 from gaphor.diagram.copypaste import deserialize, serialize
 from gaphor.event import (
@@ -65,16 +64,11 @@ class ActionStack:
 
     @transactional
     def execute(self):
-        self._actions.reverse()
+        self._actions.sort(key=lambda fn: fn.__name__)
 
         for act in self._actions:
-            logger.debug("%s", act.__doc__)
-            try:
-                act()
-            except Exception:
-                logger.error(
-                    "Error while undoing action %s", act.__doc__, exc_info=True
-                )
+            logger.debug(act.__doc__)
+            act()
 
 
 class UndoManagerStateChanged(ServiceEvent):
@@ -293,8 +287,6 @@ class UndoManager(Service, ActionProvider):
         self.event_manager.subscribe(self.undo_reversible_event)
         self.event_manager.subscribe(self.undo_create_element_event)
         self.event_manager.subscribe(self.undo_delete_element_event)
-        self.event_manager.subscribe(self.undo_create_diagram_item_event)
-        self.event_manager.subscribe(self.undo_delete_diagram_item_event)
         self.event_manager.subscribe(self.undo_attribute_change_event)
         self.event_manager.subscribe(self.undo_association_set_event)
         self.event_manager.subscribe(self.undo_association_add_event)
@@ -307,8 +299,6 @@ class UndoManager(Service, ActionProvider):
         self.event_manager.unsubscribe(self.undo_reversible_event)
         self.event_manager.unsubscribe(self.undo_create_element_event)
         self.event_manager.unsubscribe(self.undo_delete_element_event)
-        self.event_manager.unsubscribe(self.undo_create_diagram_item_event)
-        self.event_manager.unsubscribe(self.undo_delete_diagram_item_event)
         self.event_manager.unsubscribe(self.undo_attribute_change_event)
         self.event_manager.unsubscribe(self.undo_association_set_event)
         self.event_manager.unsubscribe(self.undo_association_add_event)
@@ -318,81 +308,64 @@ class UndoManager(Service, ActionProvider):
     def undo_reversible_event(self, event: RevertibeEvent):
         element_id = event.element.id
 
-        def _undo_reversible_event():
+        def b_undo_reversible_event():
             element = self.deep_lookup(element_id)
             event.revert(element)
 
-        _undo_reversible_event.__doc__ = (
+        b_undo_reversible_event.__doc__ = (
             f"Reverse event {event.__class__.__name__} for element {event.element}."
         )
 
         self.add_undo_action(
-            _undo_reversible_event, requires_transaction=event.requires_transaction
+            b_undo_reversible_event, requires_transaction=event.requires_transaction
         )
 
     @event_handler(ElementCreated)
     def undo_create_element_event(self, event: ElementCreated):
         element_id = event.element.id
 
-        def _undo_create_event():
+        def d_undo_create_event():
             element = self.deep_lookup(element_id)
             element.unlink()
 
-        _undo_create_event.__doc__ = f"Undo create element {event.element}."
+        d_undo_create_event.__doc__ = f"Undo create element {event.element}."
         del event
 
-        self.add_undo_action(_undo_create_event)
+        self.add_undo_action(d_undo_create_event)
 
     @event_handler(ElementDeleted)
     def undo_delete_element_event(self, event: ElementDeleted):
         element_type = type(event.element)
         element_id = event.element.id
 
-        def _undo_delete_event():
-            element = self.element_factory.create_as(element_type, element_id)
-            self.event_manager.handle(ElementCreated(self.element_factory, element))
+        if isinstance(event.element, Presentation):
+            diagram_id = event.diagram.id
+            data = {}
 
-        _undo_delete_event.__doc__ = f"Recreate element {element_type} ({element_id})."
+            def save_func(name, value):
+                data[name] = serialize(value)
+
+            event.element.save(save_func)
+
+            def b_undo_delete_event():
+                diagram: Diagram = self.deep_lookup(diagram_id)  # type: ignore[assignment]
+                element = diagram.create_as(element_type, element_id)
+                for name, ser in data.items():
+                    for value in deserialize(ser, lambda ref: None):
+                        element.load(name, value)
+
+            undo_delete_event = b_undo_delete_event
+        else:
+
+            def a_undo_delete_event():
+                self.element_factory.create_as(element_type, element_id)
+
+            undo_delete_event = a_undo_delete_event
+
+        undo_delete_event.__doc__ = f"Recreate element {element_type} ({element_id})."
         del event
 
-        self.add_undo_action(_undo_delete_event)
-
-    @event_handler(DiagramItemCreated)
-    def undo_create_diagram_item_event(self, event: DiagramItemCreated):
-        element_id = event.element.id
-
-        def _undo_create_event():
-            element = self.deep_lookup(element_id)
-            element.unlink()
-
-        _undo_create_event.__doc__ = f"Undo create diagram item {event.element}."
-        del event
-
-        self.add_undo_action(_undo_create_event)
-
-    @event_handler(DiagramItemDeleted)
-    def undo_delete_diagram_item_event(self, event: DiagramItemDeleted):
-        diagram_id = event.diagram.id
-        element_type = type(event.element)
-        element_id = event.element.id
-        data = {}
-
-        def save_func(name, value):
-            data[name] = serialize(value)
-
-        event.element.save(save_func)
-
-        def _undo_delete_event():
-            diagram: Diagram = self.deep_lookup(diagram_id)  # type: ignore[assignment]
-            element = diagram.create_as(element_type, element_id)
-            for name, ser in data.items():
-                for value in deserialize(ser, lambda ref: None):
-                    element.load(name, value)
-
-        _undo_delete_event.__doc__ = f"Undo delete diagram item {event.element}."
-        del event
-
-        self.add_undo_action(_undo_delete_event)
+        self.add_undo_action(undo_delete_event)
 
     @event_handler(AttributeUpdated)
     def undo_attribute_change_event(self, event: AttributeUpdated):
@@ -400,16 +373,16 @@ class UndoManager(Service, ActionProvider):
         element_id = event.element.id
         value = event.old_value
 
-        def _undo_attribute_change_event():
+        def c_undo_attribute_change_event():
             element = self.deep_lookup(element_id)
             attribute._set(element, value)
 
-        _undo_attribute_change_event.__doc__ = (
+        c_undo_attribute_change_event.__doc__ = (
             f"Revert {event.element}.{attribute.name} to {value}."
         )
         del event
 
-        self.add_undo_action(_undo_attribute_change_event)
+        self.add_undo_action(c_undo_attribute_change_event)
 
     @event_handler(AssociationSet)
     def undo_association_set_event(self, event: AssociationSet):
@@ -419,17 +392,17 @@ class UndoManager(Service, ActionProvider):
         element_id = event.element.id
         value_id = event.old_value and event.old_value.id
 
-        def _undo_association_set_event():
+        def c_undo_association_set_event():
             element = self.deep_lookup(element_id)
             value = value_id and self.deep_lookup(value_id)
             association._set(element, value, from_opposite=True)
 
-        _undo_association_set_event.__doc__ = (
+        c_undo_association_set_event.__doc__ = (
             f"Revert {event.element}.{association.name} to {event.old_value}."
         )
         del event
 
-        self.add_undo_action(_undo_association_set_event)
+        self.add_undo_action(c_undo_association_set_event)
 
     @event_handler(AssociationAdded)
     def undo_association_add_event(self, event: AssociationAdded):
@@ -439,17 +412,17 @@ class UndoManager(Service, ActionProvider):
         element_id = event.element.id
         value_id = event.new_value.id
 
-        def _undo_association_add_event():
+        def c_undo_association_add_event():
             element = self.deep_lookup(element_id)
             value = self.deep_lookup(value_id)
             association._del(element, value, from_opposite=True)
 
-        _undo_association_add_event.__doc__ = (
+        c_undo_association_add_event.__doc__ = (
             f"{event.element}.{association.name} delete {event.new_value}."
         )
         del event
 
-        self.add_undo_action(_undo_association_add_event)
+        self.add_undo_action(c_undo_association_add_event)
 
     @event_handler(AssociationDeleted)
     def undo_association_delete_event(self, event: AssociationDeleted):
@@ -459,14 +432,14 @@ class UndoManager(Service, ActionProvider):
         element_id = event.element.id
         value_id = event.old_value.id
 
-        def _undo_association_delete_event():
+        def c_undo_association_delete_event():
             element = self.deep_lookup(element_id)
             value = self.deep_lookup(value_id)
             association._set(element, value, from_opposite=True)
 
-        _undo_association_delete_event.__doc__ = (
+        c_undo_association_delete_event.__doc__ = (
             f"{event.element}.{association.name} add {event.old_value}."
         )
         del event
 
-        self.add_undo_action(_undo_association_delete_event)
+        self.add_undo_action(c_undo_association_delete_event)
