@@ -7,32 +7,34 @@ import sys
 from pathlib import Path
 from typing import Optional
 
-import gi
+from gi.repository import Gdk, Gio, GLib, Gtk
 
-from gaphor.application import Application
+from gaphor.application import Application, Session
 from gaphor.core import event_handler
-from gaphor.event import ApplicationShutdown, SessionCreated
+from gaphor.event import ActiveSessionChanged, ApplicationShutdown, SessionCreated
 from gaphor.ui.actiongroup import apply_application_actions
-
-# fmt: off
-gi.require_version("Gtk", "3.0")  # noqa: isort:skip
-gi.require_version("Gdk", "3.0")  # noqa: isort:skip
-from gi.repository import GLib, Gdk, Gio, Gtk  # noqa: isort:skip
 from gaphor.ui.macosshim import macos_init
 
-# fmt: on
-
-
 APPLICATION_ID = "org.gaphor.Gaphor"
 
 
-icon_theme = Gtk.IconTheme.get_default()
+icon_theme = (
+    Gtk.IconTheme.get_default()
+    if Gtk.get_major_version() == 3
+    else Gtk.IconTheme.get_for_display(Gdk.Display.get_default())
+)
 if sys.version_info >= (3, 9):
     path: Path = importlib.resources.files("gaphor") / "ui" / "icons"
-    icon_theme.append_search_path(str(path))
+    if Gtk.get_major_version() == 3:
+        icon_theme.append_search_path(str(path))
+    else:
+        icon_theme.add_search_path(str(path))
 else:
     with importlib.resources.path("gaphor.ui", "icons") as path:
-        icon_theme.append_search_path(str(path))
+        if Gtk.get_major_version() == 3:
+            icon_theme.append_search_path(str(path))
+        else:
+            icon_theme.add_search_path(str(path))
 
 LOG_FORMAT = "%(name)s %(levelname)s %(message)s"
 
@@ -85,9 +87,18 @@ def run(args):
 
         @event_handler(SessionCreated)
         def on_session_created(event):
+            event_manager = event.session.get_service("event_manager")
+            event_manager.subscribe(on_session_changed)
             main_window = event.session.get_service("main_window")
             main_window.open(gtk_app)
 
+        @event_handler(ActiveSessionChanged)
+        def on_session_changed(event):
+            if isinstance(event.service, Session):
+                main_window = event.service.get_service("main_window")
+                if main_window.window:
+                    main_window.window.present_with_time(Gdk.CURRENT_TIME)
+
         @event_handler(ApplicationShutdown)
         def on_quit(event):
             gtk_app.quit()
@@ -107,19 +118,16 @@ def run(args):
     def app_activate(gtk_app):
         assert application
         if not application.has_sessions():
-            app_file_manager = application.get_service("app_file_manager")
-            app_file_manager.new()
+            application.new_session()
 
     def app_open(gtk_app, files, n_files, hint):
         # appfilemanager should take care of this:
         assert application
-        app_file_manager = application.get_service("app_file_manager")
         if hint == "__new__":
-            app_file_manager.new()
+            application.new_session()
         else:
-            assert n_files == 1
             for file in files:
-                app_file_manager.load(file.get_path())
+                application.new_session(filename=file.get_path())
 
     gtk_app = Gtk.Application(
         application_id=APPLICATION_ID, flags=Gio.ApplicationFlags.HANDLES_OPEN
