@@ -1,45 +1,32 @@
 import functools
 import importlib
 import logging
-from typing import Dict, Optional, Sequence, Tuple
+from typing import Dict, Optional
 
 from gaphas.guide import GuidePainter
-from gaphas.painter import (
-    BoundingBoxPainter,
-    FreeHandPainter,
-    HandlePainter,
-    PainterChain,
-)
+from gaphas.painter import FreeHandPainter, HandlePainter, PainterChain
 from gaphas.segment import LineSegmentPainter
 from gaphas.tool.rubberband import RubberbandPainter, RubberbandState
 from gaphas.view import GtkView
-from gi.repository import Gdk, GdkPixbuf, GLib, Gtk
+from gi.repository import Gdk, GdkPixbuf, Gtk
 
 from gaphor import UML
-from gaphor.core import action, event_handler, gettext
+from gaphor.core import event_handler, gettext
 from gaphor.core.modeling import StyleSheet
 from gaphor.core.modeling.diagram import StyledDiagram
 from gaphor.core.modeling.event import AttributeUpdated, ElementDeleted
-from gaphor.diagram.diagramtoolbox import ToolDef
+from gaphor.diagram.diagramtoolbox import tooliter
 from gaphor.diagram.diagramtools import apply_default_tool_set, apply_placement_tool_set
 from gaphor.diagram.diagramtools.placement import create_item
-from gaphor.diagram.event import DiagramItemPlaced
 from gaphor.diagram.painter import ItemPainter
 from gaphor.diagram.selection import Selection
 from gaphor.diagram.support import get_diagram_item
 from gaphor.transaction import Transaction
-from gaphor.ui.actiongroup import create_action_group
-from gaphor.ui.event import DiagramSelectionChanged, Notification
+from gaphor.ui.event import DiagramSelectionChanged, Notification, ToolSelected
 
 log = logging.getLogger(__name__)
 
 
-def tooliter(toolbox_actions: Sequence[Tuple[str, Sequence[ToolDef]]]):
-    """Iterate toolbox items, regardless of section headers."""
-    for name, section in toolbox_actions:
-        yield from section
-
-
 @functools.lru_cache(maxsize=1)
 def placement_icon_base():
     with importlib.resources.path("gaphor.ui", "placement-icon-base.png") as f:
@@ -48,37 +35,74 @@ def placement_icon_base():
 
 GtkView.set_css_name("diagramview")
 
-_placement_pixbuf_map: Dict[str, GdkPixbuf.Pixbuf] = {}
+
+if Gtk.get_major_version() == 3:
+    _placement_pixbuf_map: Dict[str, GdkPixbuf.Pixbuf] = {}
+
+    def get_placement_cursor(display, icon_name):
+        if icon_name in _placement_pixbuf_map:
+            pixbuf = _placement_pixbuf_map[icon_name]
+        else:
+            pixbuf = placement_icon_base().copy()
+            icon = Gtk.IconTheme.get_default().load_icon(icon_name, 24, 0)
+            icon.copy_area(
+                0,
+                0,
+                icon.get_width(),
+                icon.get_height(),
+                pixbuf,
+                9,
+                15,
+            )
+            _placement_pixbuf_map[icon_name] = pixbuf
+        return Gdk.Cursor.new_from_pixbuf(display, pixbuf, 1, 1)
 
 
-def get_placement_cursor(display, icon_name):
-    if icon_name in _placement_pixbuf_map:
-        pixbuf = _placement_pixbuf_map[icon_name]
-    else:
-        pixbuf = placement_icon_base().copy()
-        icon = Gtk.IconTheme.get_default().load_icon(icon_name, 24, 0)
-        icon.copy_area(
-            0,
-            0,
-            icon.get_width(),
-            icon.get_height(),
-            pixbuf,
-            9,
-            15,
-        )
-        _placement_pixbuf_map[icon_name] = pixbuf
-    return Gdk.Cursor.new_from_pixbuf(display, pixbuf, 1, 1)
+else:
+    _placement_texture_map: Dict[str, Gdk.Texture] = {}
+
+    def get_placement_cursor(display, icon_name):
+        if display is None:
+            display = Gdk.Display.get_default()
+        if icon_name in _placement_texture_map:
+            texture = _placement_texture_map[icon_name]
+        else:
+            pixbuf = placement_icon_base().copy()
+            theme_icon = Gtk.IconTheme.get_for_display(display).lookup_icon(
+                icon_name,
+                None,
+                24,
+                1,
+                Gtk.TextDirection.NONE,
+                Gtk.IconLookupFlags.FORCE_SYMBOLIC,
+            )
+            icon = GdkPixbuf.Pixbuf.new_from_file_at_scale(
+                theme_icon.get_file().get_path(), 32, 32, True
+            )
+            icon.copy_area(
+                0,
+                0,
+                icon.get_width(),
+                icon.get_height(),
+                pixbuf,
+                9,
+                15,
+            )
+            texture = Gdk.Texture.new_for_pixbuf(pixbuf)
+            _placement_texture_map[icon_name] = texture
+        return Gdk.Cursor.new_from_texture(texture, 1, 1)
 
 
 class DiagramPage:
 
-    VIEW_TARGET_STRING = 0
-    VIEW_TARGET_ELEMENT_ID = 1
-    VIEW_TARGET_TOOLBOX_ACTION = 2
-    VIEW_DND_TARGETS = [
-        Gtk.TargetEntry.new("gaphor/element-id", 0, VIEW_TARGET_ELEMENT_ID),
-        Gtk.TargetEntry.new("gaphor/toolbox-action", 0, VIEW_TARGET_TOOLBOX_ACTION),
-    ]
+    if Gtk.get_major_version() == 3:
+        VIEW_TARGET_STRING = 0
+        VIEW_TARGET_ELEMENT_ID = 1
+        VIEW_TARGET_TOOLBOX_ACTION = 2
+        VIEW_DND_TARGETS = [
+            Gtk.TargetEntry.new("gaphor/element-id", 0, VIEW_TARGET_ELEMENT_ID),
+            Gtk.TargetEntry.new("gaphor/toolbox-action", 0, VIEW_TARGET_TOOLBOX_ACTION),
+        ]
 
     def __init__(
         self, diagram, event_manager, element_factory, properties, modeling_language
@@ -97,7 +121,7 @@ class DiagramPage:
 
         self.event_manager.subscribe(self._on_element_delete)
         self.event_manager.subscribe(self._on_style_sheet_updated)
-        self.event_manager.subscribe(self._on_diagram_item_placed)
+        self.event_manager.subscribe(self._on_tool_selected)
 
     title = property(lambda s: s.diagram and s.diagram.name or gettext("<None>"))
 
@@ -114,12 +138,17 @@ class DiagramPage:
         """
         assert self.diagram
 
-        view = GtkView(model=self.diagram, selection=Selection())
-        view.drag_dest_set(
-            Gtk.DestDefaults.ALL,
-            DiagramPage.VIEW_DND_TARGETS,
-            Gdk.DragAction.MOVE | Gdk.DragAction.COPY | Gdk.DragAction.LINK,
-        )
+        view = GtkView(selection=Selection())
+        if Gtk.get_major_version() == 3:
+            view.drag_dest_set(
+                Gtk.DestDefaults.ALL,
+                DiagramPage.VIEW_DND_TARGETS,
+                Gdk.DragAction.MOVE | Gdk.DragAction.COPY | Gdk.DragAction.LINK,
+            )
+        else:
+            # TODO: Gtk4 - use controllers DragSource and DropTarget
+            pass
+
         self.diagram_css = Gtk.CssProvider.new()
         view.get_style_context().add_provider(
             self.diagram_css, Gtk.STYLE_PROVIDER_PRIORITY_USER
@@ -127,22 +156,25 @@ class DiagramPage:
 
         scrolled_window = Gtk.ScrolledWindow()
         scrolled_window.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
-        scrolled_window.set_shadow_type(Gtk.ShadowType.IN)
-        scrolled_window.add(view)
-        scrolled_window.show_all()
-        self.widget = scrolled_window
+        if Gtk.get_major_version() == 3:
+            scrolled_window.add(view)
+            scrolled_window.show_all()
+            view.connect("drag-data-received", self._on_drag_data_received)
+        else:
+            scrolled_window.set_child(view)
 
         view.selection.add_handler(self._on_view_selection_changed)
-        view.connect("drag-data-received", self._on_drag_data_received)
 
         self.view = view
-
-        self.widget.action_group = create_action_group(self, "diagram")
+        self.widget = scrolled_window
 
         self.select_tool("toolbox-pointer")
 
         self.set_drawing_style()
 
+        # Set model only after the painters are set
+        view.model = self.diagram
+
         return self.widget
 
     def get_tool_def(self, tool_name):
@@ -182,6 +214,10 @@ class DiagramPage:
             if t.id == tool_name
         ).icon_name
 
+    @event_handler(ToolSelected)
+    def _on_tool_selected(self, event: ToolSelected):
+        self.select_tool(event.tool_name)
+
     @event_handler(ElementDeleted)
     def _on_element_delete(self, event: ElementDeleted):
         if event.element is self.diagram:
@@ -202,70 +238,35 @@ class DiagramPage:
         Do the same thing that would be done if Close was pressed.
         """
         assert self.widget
-        self.widget.destroy()
+        if Gtk.get_major_version() == 3:
+            self.widget.destroy()
+        else:
+            parent = self.widget.get_parent()
+            if parent:
+                parent.remove(self.widget)
+
         self.event_manager.unsubscribe(self._on_element_delete)
         self.event_manager.unsubscribe(self._on_style_sheet_updated)
-        self.event_manager.unsubscribe(self._on_diagram_item_placed)
+        self.event_manager.unsubscribe(self._on_tool_selected)
         self.view = None
 
-    @action(
-        name="diagram.zoom-in",
-        shortcut="<Primary>plus",
-    )
-    def zoom_in(self):
-        assert self.view
-        self.view.zoom(1.2)
-
-    @action(
-        name="diagram.zoom-out",
-        shortcut="<Primary>minus",
-    )
-    def zoom_out(self):
-        assert self.view
-        self.view.zoom(1 / 1.2)
-
-    @action(
-        name="diagram.zoom-100",
-        shortcut="<Primary>0",
-    )
-    def zoom_100(self):
-        assert self.view
-        zx = self.view.matrix[0]
-        self.view.zoom(1 / zx)
-
-    @action(
-        name="diagram.select-all",
-        shortcut="<Primary>a",
-    )
-    def select_all(self):
-        assert self.view
-        if self.view.has_focus():
-            self.view.selection.select_items(*self.view.model.get_all_items())
-
-    @action(name="diagram.unselect-all", shortcut="<Primary><Shift>a")
-    def unselect_all(self):
-        assert self.view
-        if self.view.has_focus():
-            self.view.selection.unselect_all()
-
-    @action(name="diagram.select-tool", state="toolbox-pointer")
     def select_tool(self, tool_name: str):
         if self.view:
             self.apply_tool_set(tool_name)
             icon_name = self.get_tool_icon_name(tool_name)
-            window = self.view.get_window()
-            if icon_name and window:
-                window.set_cursor(get_placement_cursor(window.get_display(), icon_name))
-            elif window:
-                window.set_cursor(None)
-
-    @event_handler(DiagramItemPlaced)
-    def _on_diagram_item_placed(self, event):
-        assert self.widget
-        if self.properties.get("reset-tool-after-create", True):
-            self.widget.action_group.actions.lookup_action("select-tool").activate(
-                GLib.Variant.new_string("toolbox-pointer")
-            )
+            if Gtk.get_major_version() == 3:
+                window = self.view.get_window()
+                if icon_name and window:
+                    window.set_cursor(
+                        get_placement_cursor(window.get_display(), icon_name)
+                    )
+                elif window:
+                    window.set_cursor(None)
+            else:
+                if icon_name:
+                    self.view.set_cursor(get_placement_cursor(None, icon_name))
+                else:
+                    self.view.set_cursor(None)
 
     def set_drawing_style(self):
         """Set the drawing style for the diagram based on the active style
@@ -277,9 +278,9 @@ class DiagramPage:
 
         bg = style.get("background-color")
         self.diagram_css.load_from_data(
-            f"diagramview {{ background-color: rgba({int(255*bg[0])}, {int(255*bg[1])}, {int(255*bg[2])}, {bg[3]}) }}".encode()
+            f"diagramview {{ background-color: rgba({int(255*bg[0])}, {int(255*bg[1])}, {int(255*bg[2])}, {bg[3]}); }}".encode()
             if bg
-            else "".encode()
+            else b""
         )
 
         view = self.view
@@ -290,6 +291,7 @@ class DiagramPage:
         if sloppiness:
             item_painter = FreeHandPainter(item_painter, sloppiness=sloppiness)
 
+        view.bounding_box_painter = item_painter
         view.painter = (
             PainterChain()
             .append(item_painter)
@@ -298,11 +300,10 @@ class DiagramPage:
             .append(GuidePainter(view))
             .append(RubberbandPainter(self.rubberband_state))
         )
-        view.bounding_box_painter = BoundingBoxPainter(item_painter)
 
-        view.queue_redraw()
+        view.request_update(self.diagram.get_all_items())
 
-    def _on_view_selection_changed(self):
+    def _on_view_selection_changed(self, item):
         view = self.view
         assert view
         selection = view.selection
