@@ -18,21 +18,9 @@ class DiagramSelectionChanged:
         self.selected_items = selected_items
 
 
-class ModelingLanguageChanged:
-    def __init__(self, modeling_language):
-        self.modeling_language = modeling_language
-
-
-class FileLoaded:
-    def __init__(self, service, filename=None):
-        self.service = service
-        self.filename = filename
-
-
-class FileSaved:
-    def __init__(self, service, filename=None):
-        self.service = service
-        self.filename = filename
+class ToolSelected:
+    def __init__(self, tool_name):
+        self.tool_name = tool_name
 
 
 class Notification:
