@@ -1,18 +1,26 @@
 """The file service is responsible for loading and saving the user data."""
 
 import logging
+from queue import Queue
 
-from gi.repository import Gtk
+from gaphas.decorators import g_async
+from gi.repository import GLib, Gtk
 
+from gaphor import UML
 from gaphor.abc import ActionProvider, Service
 from gaphor.core import action, event_handler, gettext
-from gaphor.event import SessionShutdown, SessionShutdownRequested
+from gaphor.core.modeling.stylesheet import StyleSheet
+from gaphor.event import (
+    ModelLoaded,
+    ModelSaved,
+    SessionCreated,
+    SessionShutdown,
+    SessionShutdownRequested,
+)
 from gaphor.storage import storage, verify
 from gaphor.storage.xmlwriter import XMLWriter
 from gaphor.ui.errorhandler import error_handler
-from gaphor.ui.event import FileLoaded, FileSaved
 from gaphor.ui.filedialog import GAPHOR_FILTER, save_file_dialog
-from gaphor.ui.gidlethread import GIdleThread, Queue
 from gaphor.ui.questiondialog import QuestionDialog
 from gaphor.ui.statuswindow import StatusWindow
 
@@ -40,6 +48,18 @@ def error_message(e):
     ).format(exc=str(e))
 
 
+def load_default_model(element_factory):
+    element_factory.flush()
+    with element_factory.block_events():
+        element_factory.create(StyleSheet)
+        model = element_factory.create(UML.Package)
+        model.name = gettext("New model")
+        diagram = element_factory.create(UML.Diagram)
+        diagram.element = model
+        diagram.name = gettext("main")
+    element_factory.model_ready()
+
+
 class FileManager(Service, ActionProvider):
     """The file service, responsible for loading and saving Gaphor models."""
 
@@ -55,10 +75,12 @@ class FileManager(Service, ActionProvider):
         self._filename = None
 
         event_manager.subscribe(self._on_session_shutdown_request)
+        event_manager.subscribe(self._on_session_created)
 
     def shutdown(self):
         """Called when shutting down the file manager service."""
         self.event_manager.unsubscribe(self._on_session_shutdown_request)
+        self.event_manager.unsubscribe(self._on_session_created)
 
     def get_filename(self):
         """Return the current file name.
@@ -87,8 +109,9 @@ class FileManager(Service, ActionProvider):
         GIdleThread which executes the load generator.  If loading is
         successful, the filename is set.
         """
-
-        queue = Queue()
+        # First claim file name, so any other files will be opened in a different session
+        self.filename = filename
+        queue: Queue[int] = Queue(0)
         status_window = StatusWindow(
             gettext("Loading..."),
             gettext("Loading model from {filename}").format(filename=filename),
@@ -96,33 +119,37 @@ class FileManager(Service, ActionProvider):
             queue=queue,
         )
 
-        try:
-            loader = storage.load_generator(
-                filename.encode("utf-8"), self.element_factory, self.modeling_language
-            )
-            worker = GIdleThread(loader, queue)
+        # Use low prio, so screen updates do happen
+        @g_async(priority=GLib.PRIORITY_DEFAULT_IDLE)
+        def async_loader():
+            try:
+                for percentage in storage.load_generator(
+                    filename.encode("utf-8"),
+                    self.element_factory,
+                    self.modeling_language,
+                ):
+                    queue.put(percentage)
+                    yield percentage
 
-            worker.start()
-            worker.wait()
+                self.event_manager.handle(ModelLoaded(self, filename))
+            except Exception:
+                self.filename = None
+                error_handler(
+                    message=gettext("Unable to open model “{filename}”.").format(
+                        filename=filename
+                    ),
+                    secondary_message=gettext(
+                        "This file does not contain a valid Gaphor model."
+                    ),
+                    window=self.main_window.window,
+                )
+                load_default_model(self.element_factory)
+                raise
+            finally:
+                status_window.destroy()
 
-            if worker.error:
-                worker.reraise()
-
-            self.filename = filename
-            self.event_manager.handle(FileLoaded(self, filename))
-        except Exception:
-            error_handler(
-                message=gettext("Unable to open model “{filename}”.").format(
-                    filename=filename
-                ),
-                secondary_message=gettext(
-                    "This file does not contain a valid Gaphor model."
-                ),
-                window=self.main_window.window,
-            )
-            raise
-        finally:
-            status_window.destroy()
+        for _ in async_loader():
+            pass
 
     def verify_orphans(self):
         """Verify that no orphaned elements are saved.
@@ -167,36 +194,40 @@ class FileManager(Service, ActionProvider):
         self.verify_orphans()
 
         main_window = self.main_window
-        queue = Queue()
+        queue: Queue[int] = Queue(0)
         status_window = StatusWindow(
             gettext("Saving..."),
             gettext("Saving model to {filename}").format(filename=filename),
             parent=main_window.window,
             queue=queue,
         )
-        try:
-            with open(filename.encode("utf-8"), "w") as out:
-                saver = storage.save_generator(XMLWriter(out), self.element_factory)
-                worker = GIdleThread(saver, queue)
-                worker.start()
-                worker.wait()
 
-            if worker.error:
-                worker.reraise()
+        @g_async(priority=GLib.PRIORITY_DEFAULT_IDLE)
+        def async_saver():
+            try:
+                with open(filename.encode("utf-8"), "w") as out:
+                    for percentage in storage.save_generator(
+                        XMLWriter(out), self.element_factory
+                    ):
+                        queue.put(percentage)
+                        yield
 
-            self.filename = filename
-            self.event_manager.handle(FileSaved(self, filename))
-        except Exception as e:
-            error_handler(
-                message=gettext("Unable to save model “{filename}”.").format(
-                    filename=filename
-                ),
-                secondary_message=error_message(e),
-                window=self.main_window.window,
-            )
-            raise
-        finally:
-            status_window.destroy()
+                self.filename = filename
+                self.event_manager.handle(ModelSaved(self, filename))
+            except Exception as e:
+                error_handler(
+                    message=gettext("Unable to save model “{filename}”.").format(
+                        filename=filename
+                    ),
+                    secondary_message=error_message(e),
+                    window=self.main_window.window,
+                )
+                raise
+            finally:
+                status_window.destroy()
+
+        for _ in async_saver():
+            pass
 
     @action(name="file-save", shortcut="<Primary>s")
     def action_save(self):
@@ -236,6 +267,13 @@ class FileManager(Service, ActionProvider):
 
         return False
 
+    @event_handler(SessionCreated)
+    def _on_session_created(self, event: SessionCreated):
+        if event.filename:
+            self.load(event.filename)
+        else:
+            load_default_model(self.element_factory)
+
     @event_handler(SessionShutdownRequested)
     def _on_session_shutdown_request(self, event):
         """Ask user to close window if the model has changed.
@@ -248,28 +286,7 @@ class FileManager(Service, ActionProvider):
             self.event_manager.handle(SessionShutdown(self))
 
         if self.main_window.model_changed:
-            dialog = Gtk.MessageDialog(
-                self.main_window.window,
-                Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
-                Gtk.MessageType.WARNING,
-            )
-            dialog.props.text = gettext("Save changed to your model before closing?")
-            dialog.props.secondary_text = gettext(
-                "If you close without saving, your changes will be discarded."
-            )
-
-            dialog.add_buttons(
-                gettext("Close _without saving"),
-                Gtk.ResponseType.REJECT,
-                Gtk.STOCK_CANCEL,
-                Gtk.ResponseType.CANCEL,
-                Gtk.STOCK_SAVE,
-                Gtk.ResponseType.YES,
-            )
-            dialog.set_default_response(Gtk.ResponseType.YES)
-            response = dialog.run()
-            dialog.destroy()
-
+            response = save_changes_before_closing_dialog(self.main_window.window)
             if response == Gtk.ResponseType.YES:
                 saved = self.action_save()
                 if saved:
@@ -278,3 +295,29 @@ class FileManager(Service, ActionProvider):
                 confirm_shutdown()
         else:
             confirm_shutdown()
+
+
+def save_changes_before_closing_dialog(window):
+    dialog = Gtk.MessageDialog(
+        window,
+        Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
+        Gtk.MessageType.WARNING,
+    )
+    dialog.props.text = gettext("Save changed to your model before closing?")
+    dialog.props.secondary_text = gettext(
+        "If you close without saving, your changes will be discarded."
+    )
+
+    dialog.add_buttons(
+        gettext("Close _without saving"),
+        Gtk.ResponseType.REJECT,
+        Gtk.STOCK_CANCEL,
+        Gtk.ResponseType.CANCEL,
+        Gtk.STOCK_SAVE,
+        Gtk.ResponseType.YES,
+    )
+    dialog.set_default_response(Gtk.ResponseType.YES)
+    response = dialog.run()
+    dialog.destroy()
+
+    return response
